/* 
   Replication Server - RepAgentID V1.0, November 6, 2001 

   Written by: 
      Alan Fedoruk                                           -- amf673@mail.usask.ca
      MADMUC Lab                                             -- http://bistrica.usask.ca/madmuc/index.htm
      Dept. of Computer Science, University of Saskatchewan. -- http://www.cs.usask.ca

   Copyright (c) 2001 Alan M. Fedoruk
*/

package repserver;

import fipaos.ont.fipa.fipaman.*;
import fipaos.parser.*;

import java.util.Date; 


/** RepAgentID is an extension of AgentID to handle replication issues. 

    RepAgentID adds a data field to store the last time this 
    replicant either set or sent its state and it adds a boolean field 
    to indicate that the last ping failed or not. 
*/
public class RepAgentID extends AgentID 
{
   /** The time this replicant was last updated with a state. */ 
   private Date _update_time;

   /** Did the last ping suceeed? */ 
   private boolean _ping;
	

   /** Constructs a blank RepAgentID.  
   */
   public RepAgentID() 
   {
     super(); 
     _update_time = null; 
     _ping = true; 
   } 


   /** Constructs a RepAgentID from an AID string. See AgentID.  
   */
   public RepAgentID( String aid) throws ParserException 
   {
      super(aid); 
     _update_time = null; 
     _ping = true; 
   } 


   /** Constructs a RepAgentID from an AgentID 
   */
   public RepAgentID( AgentID aid) 
   {
      super(aid); 
     _update_time = null; 
     _ping = true; 
   } 
 

   /** set the update time 
       @param u date/time to set. 
   */
   public void setUpdate(Date u) 
   {
      _update_time = u;
   } 


   /** return the update time 
   */
   public Date getUpdate() 
   {
       return _update_time; 
   } 


   /** set the ping flag. 
       @parm p t/f to set. 
   */
   public void setPing(boolean p) 
   {
      _ping = p;
   } 


   /** return the current ping flag
   */
   public boolean getPing() 
   {
      return _ping;
   } 

} // RepAgentID
