/* 
   Test Replication Server - TestAgent V1.0, November 6, 2001 

   Written by: 
      Alan Fedoruk                                           -- amf673@mail.usask.ca
      MADMUC Lab                                             -- http://bistrica.usask.ca/madmuc/index.htm
      Dept. of Computer Science, University of Saskatchewan. -- http://www.cs.usask.ca

   Copyright (c) 2001 Alan M. Fedoruk
*/


  
package testConv;

// Import the fipa-os classes
import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.mts.*;
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.parser.acl.ACLMessage;

// Import registration exception classes
import fipaos.platform.ams.AMSRegistrationException;
import fipaos.platform.df.DFRegistrationException;

import fipaos.util.DIAGNOSTICS;

import repserver.*;

import java.util.*;

/** TestAgent exercises the replication server.
    It sends requests to the replication server to create two replicant groups, 
    then it sends requests to the counter replicant group. 
 */
public class Receiver extends FIPAOSAgent 
{
   /** A RepServer that this TestAgent can send requests to */
   private AgentID _rs_agent;

   /** This Agent type */
   private static final String AGENT_TYPE = "test-agent";

   /** Agent type to send requests to */
   private static final String RS_TYPE = RSCONSTANTS.REPSERVER; 
  

   /**
    * Agent constructor - sets up an IdleTask as the default
    * ListenerTask for incoming messages, and starts off registration
    * with the AMS and DF
    */
   public Receiver( String platform, String name, String ownership ) 
   {
      super( platform, name, ownership );
 
    
      // Instantiate a new IdleTask to receieve all incoming 
      // communications which are not associated with any-other Task

      setListenerTask( new IdleTask() );
  
      try 
      {
         registerWithAMS();
         DIAGNOSTICS.println( "Registered with AMS", this, DIAGNOSTICS.LEVEL_MAX );

      } 
      catch (AMSRegistrationException amsre)
      {
         DIAGNOSTICS.println(amsre, this, DIAGNOSTICS.LEVEL_MAX );
         String reason = amsre.getExceptionReason();
         if ( reason == null || !reason.equals(
               FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED)) 
         {
            // Shutdown nicely :)
            shutdown();
            return;
         }
      } // catch 
    
      try 
      {
         registerWithDF( AGENT_TYPE );
         DIAGNOSTICS.println("Registered with DF", this, 
               DIAGNOSTICS.LEVEL_MAX );
      } 
      catch (DFRegistrationException dfre) 
      {
         DIAGNOSTICS.println(dfre, this, DIAGNOSTICS.LEVEL_MAX );
         String reason = dfre.getExceptionReason();
         if ( reason == null || !reason.equals(
               FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED) ) 
         {

            shutdown();
            return;
         }
      
      } // end catch
    
   } // TestAgent()
  

   /**
    * Invoked when this Agent is about to be shutdown - this overrides the
    * shutdown() method in the FIPAOSAgent class to enable the TestAgentGUI
    * associated with this TestAgent to be removed
    */
   public synchronized void shutdown() 
   {
      if (registeredWithDF() == true) 
      {
         // Attempt to deregister with DF
         try 
         {

            deregisterWithDF();
            DIAGNOSTICS.println("Deregistered with DF", this, 
                  DIAGNOSTICS.LEVEL_MAX );

         } catch (DFRegistrationException dfre) 
         {
            // A problem deregistering occured..... we can obtain 
            // the reason though!
            String reason = dfre.getExceptionReason();

            DIAGNOSTICS.println(dfre + ":" + reason, this, 
                  DIAGNOSTICS.LEVEL_MAX ); 
         } // end catch
      } // if 
    
      // Check if we're registered with the AMS
      if (registeredWithAMS() == true) 
      {
         // Attempt to deregister with AMS
         try 
         {
            deregisterWithAMS();
            DIAGNOSTICS.println("Deregistered with AMS", this, 
                  DIAGNOSTICS.LEVEL_MAX ); 
         } 
         catch (AMSRegistrationException amsre) 
         {
            // A problem deregistering occured..... we can obtain the 
            // reason though!
            String reason = amsre.getExceptionReason();
     
            DIAGNOSTICS.println(amsre + ":" + reason, this, DIAGNOSTICS.LEVEL_MAX );
         } // catch
      } // if 
     

    
      // Now call shutdown in the agent shell to release the core components
      super.shutdown();

   } // shutdown()
  

   /**
    * This method is invoked when GUI should be shown - generally
    * this is invoked by the AgentLoader when the Agent is in the List
    * of running Agents - the default implementation in FIPAOSAgent does 
    * nothing.
    */

   public void activate() 
   {

   } // activate()
    

  /**
    * Send a reply message in response to the given ACL message
    *
    * @param acl The ACL object to reply to
    * @param reason Any additional message
    **/
   public boolean sendReply( ACL orig_acl, String performative, String content )
   {
      boolean done = false;
      
      try
      {
         // Duplicate ACL
         ACL acl = (ACL) orig_acl.clone();
            
         // Setup message
         acl.setPerformative( performative );
         acl.setReceiverAID( acl.getSenderAID() );
         acl.setSenderAID( getAID() );
         acl.setInReplyTo( acl.getReplyWith() );
         acl.setReplyWith( null );
            
         // Set content
         acl.setContentObject( content );
            
         // Attempt to send the agree
         forward( acl );
            
         // We're done
         done = true;
      }
      catch( Throwable t )
      {
         DIAGNOSTICS.println( "Error occured when sending agree", t, this,
         DIAGNOSTICS.LEVEL_MAX );
      }
      
      return done;
   } // sendReply()
	

public int getConvs()
   {
      return getCM().getConvCount();
   }
public void removeConv(Conversation conv)
{
  try {
   getCM().removeConversation(conv.getConversationID());
  }
  catch (Exception e)
  {
     System.out.println(e);
  }
}



   public class IdleTask extends Task 
   {


      public IdleTask() 
      {
      } 
    
    
      protected void startTask() 
      {
      } // StartTask()
    
    
      public void handleRequest(Conversation c) 
      {
         ACL acl = c.getACL( c.getLatestMessageIndex());
         sendReply(acl, FIPACONSTANTS.AGREE, "");
         sendReply(acl, FIPACONSTANTS.INFORM, "");
removeConv(c);
      } 
    
   } // IdleTask()



   /**
   * Test-harness
   */
   public static void main( String args[] ) 
   {
      if ( args == null || args.length < 2 ) 
      {
         System.out.println( "Usage: java testRS.TestAgent <platform.profile> <name>" );
         return;
      }
      new Receiver( args[0], args[1], "FIPA-OS" );

   } // main()

} // TestAgent
