/* 
   Test Replication Server - TestAgent V1.0, November 6, 2001 

   Written by: 
      Alan Fedoruk                                           -- amf673@mail.usask.ca
      MADMUC Lab                                             -- http://bistrica.usask.ca/madmuc/index.htm
      Dept. of Computer Science, University of Saskatchewan. -- http://www.cs.usask.ca

   Copyright (c) 2001 Alan M. Fedoruk
*/


  
package testConv;

// Import the fipa-os classes
import fipaos.agent.*;
import fipaos.agent.conversation.*;
import fipaos.agent.task.*;
import fipaos.mts.*;
import fipaos.ont.fipa.*;
import fipaos.ont.fipa.fipaman.*;
import fipaos.parser.acl.ACLMessage;

// Import registration exception classes
import fipaos.platform.ams.AMSRegistrationException;
import fipaos.platform.df.DFRegistrationException;

import fipaos.util.DIAGNOSTICS;

import repserver.*;

import java.util.*;

/** TestAgent exercises the replication server.
    It sends requests to the replication server to create two replicant groups, 
    then it sends requests to the counter replicant group. 
 */
public class Sender extends FIPAOSAgent 
{
   /** A RepServer that this TestAgent can send requests to */
   private AgentID _rs_agent;

   /** This Agent type */
   private static final String AGENT_TYPE = "test-agent";

   /** Agent type to send requests to */
   private static final String RS_TYPE = RSCONSTANTS.REPSERVER; 
  

   /**
    * Agent constructor - sets up an IdleTask as the default
    * ListenerTask for incoming messages, and starts off registration
    * with the AMS and DF
    */
   public Sender( String platform, String name, String ownership ) 
   {
      super( platform, name, ownership );
 
      // Instantiate a new IdleTask to receieve all incoming 
      // communications which are not associated with any-other Task

      setListenerTask( new IdleTask() );
  
      try 
      {
         registerWithAMS();
         DIAGNOSTICS.println( "Registered with AMS", this, DIAGNOSTICS.LEVEL_MAX );

      } 
      catch (AMSRegistrationException amsre)
      {
         DIAGNOSTICS.println(amsre, this, DIAGNOSTICS.LEVEL_MAX );
         String reason = amsre.getExceptionReason();
         if ( reason == null || !reason.equals(
               FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED)) 
         {
            // Shutdown nicely :)
            shutdown();
            return;
         }
      } // catch 
    
      try 
      {
         registerWithDF( AGENT_TYPE );
         DIAGNOSTICS.println("Registered with DF", this, 
               DIAGNOSTICS.LEVEL_MAX );
      } 
      catch (DFRegistrationException dfre) 
      {
         DIAGNOSTICS.println(dfre, this, DIAGNOSTICS.LEVEL_MAX );
         String reason = dfre.getExceptionReason();
         if ( reason == null || !reason.equals(
               FIPAMANCONSTANTS.AGENT_ALREADY_REGISTERED) ) 
         {

            shutdown();
            return;
         }
      
      } // end catch
    
   } // TestAgent()
  

   /**
    * Invoked when this Agent is about to be shutdown - this overrides the
    * shutdown() method in the FIPAOSAgent class to enable the TestAgentGUI
    * associated with this TestAgent to be removed
    */
   public synchronized void shutdown() 
   {
      if (registeredWithDF() == true) 
      {
         // Attempt to deregister with DF
         try 
         {

            deregisterWithDF();
            DIAGNOSTICS.println("Deregistered with DF", this, 
                  DIAGNOSTICS.LEVEL_MAX );

         } catch (DFRegistrationException dfre) 
         {
            // A problem deregistering occured..... we can obtain 
            // the reason though!
            String reason = dfre.getExceptionReason();

            DIAGNOSTICS.println(dfre + ":" + reason, this, 
                  DIAGNOSTICS.LEVEL_MAX ); 
         } // end catch
      } // if 
    
      // Check if we're registered with the AMS
      if (registeredWithAMS() == true) 
      {
         // Attempt to deregister with AMS
         try 
         {
            deregisterWithAMS();
            DIAGNOSTICS.println("Deregistered with AMS", this, 
                  DIAGNOSTICS.LEVEL_MAX ); 
         } 
         catch (AMSRegistrationException amsre) 
         {
            // A problem deregistering occured..... we can obtain the 
            // reason though!
            String reason = amsre.getExceptionReason();
     
            DIAGNOSTICS.println(amsre + ":" + reason, this, DIAGNOSTICS.LEVEL_MAX );
         } // catch
      } // if 
     
    
      // Now call shutdown in the agent shell to release the core components
      super.shutdown();

   } // shutdown()
  

   /**
    * This method is invoked when GUI should be shown - generally
    * this is invoked by the AgentLoader when the Agent is in the List
    * of running Agents - the default implementation in FIPAOSAgent does 
    * nothing.
    */

   public void activate() 
   {
   } // activate()
    
 public int getConvs()
   {
      return getCM().getConvCount();
   }

public void removeConv(Conversation conv)
{
  try {
   getCM().removeConversation(conv.getConversationID());
  }
  catch (Exception e)
  {
     System.out.println(e);
  }
}


   public class IdleTask extends Task 
   {

       int n = 0;
       long starttime; 
       long currtime; 
       long etime; 
       float rate;
       String sendtoname;
       AgentID sendtoAID;


      public IdleTask() 
      {
         String myname = getAID().getName();
         String suff = myname.substring(myname.indexOf('_' )+1,myname.indexOf('_')+2);  
         sendtoname  = "receiver_" + suff + "@localap";
      } 
    
    
      protected void startTask() 
      {
         // do a df search and find receiver agents

         // Create the DFAgentDescription describing the attributes we wish
         // to constrain in our search for test-agent types
         String RS_TYPE = "test-agent";
         DFAgentDescription df_desc = new DFAgentDescription();
      
         // Create a ServiceDescription of the test-agent
         ServiceDescription sd = new ServiceDescription();
         sd.setServiceName(RS_TYPE);
         sd.setServiceType(RS_TYPE); 
         df_desc.setAgentServices( new ServiceDescription[] { sd } );
      
         // Start a search for other Agents
         searchDF( df_desc );
         System.out.println("Started DF Search");
   
      } // StartTask()
    

         

      /** This method is invoked when a DFSearchTask has completed
          succesfully.
       */
      public void DFSearchResults( DFAgentDescription descs[] ) 
      {
         // Check we've got some results - if not just return
         if( descs == null || descs.length <= 0 ) 
         {
            return;
         }
         System.out.println( "Found " + (descs.length) + " test-agents" );
        
         // find the one for us! 
         int i = 0;
         while ( i < descs.length )
         {
            if ( descs[i].getAgentID().getName().equals(sendtoname) )
            {
               sendtoAID = descs[i].getAgentID();
               break;
            }
            else 
               i++;
         }
         if ( i == descs.length) 
         {
            System.out.println("Could not find receiver");
            return;
         }
         else 
         {
            starttime = System.currentTimeMillis();
            ACL request = new ACL();
            request.setPerformative( FIPACONSTANTS.REQUEST);
            request.setSenderAID( getAID());
            request.setReceiverAID( sendtoAID);
            forward( request); 
         } // else
      
      } // DFSearchResults()

    
    
      public void handleAgree(Conversation conv) 
      {
      }


      public void handleInform(Conversation conv) 
      {
         n++; 
         if ( n % 100 == 0) { 
            currtime = System.currentTimeMillis();
            etime = currtime - starttime; 
            starttime = currtime; 
            rate = etime / 100;  
            System.out.println(" Request/Replies: " + n + " in " + etime +
                  " at " + rate + " " + getConvs() + " Conversations" );
         }
         removeConv(conv);
         ACL request = new ACL();
         request.setPerformative( FIPACONSTANTS.REQUEST);
         request.setSenderAID( getAID());
         request.setReceiverAID( sendtoAID);
         forward(request); 
      }

     
   } // IdleTask()



   /**
   * Test-harness
   */
   public static void main( String args[] ) 
   {
      if ( args == null || args.length < 2 ) 
      {
         System.out.println( "Usage: java testConv.Sender <platform.profile> <name>" );
         return;
      }
      new Sender( args[0], args[1], "FIPA-OS" );

   } // main()

} // TestAgent
